@@ -31,6 +31,7 @@ from panda3d.core import (
 
 from nx3d import utils
 from nx3d.types import Pos3
+from nx3d.utils import get_pos_scale
 
 DO_LS = False
 logger.remove()
@@ -41,6 +42,8 @@ elif os.environ.get("DEBUG"):
     logger.add(sys.stderr, level="DEBUG")
 elif os.environ.get("INFO"):
     logger.add(sys.stderr, level="INFO")
+else:
+    logger.add(sys.stderr, level="WARNING")
 
 
 loadPrcFileData("", "background-color .03")
@@ -172,7 +175,7 @@ class Nx3D(ShowBase):
         **kwargs,
     ):
         ShowBase.__init__(self, **kwargs)
-        logger.info("")
+        logger.info(graph)
         self.g = graph
         self.time_elapsed = 0.0
         self._latest_key = None
@@ -189,6 +192,7 @@ class Nx3D(ShowBase):
             autolabel_nodes = autolabel_edges = True
         self._init_positions(pos)
         self._init_render_attrs(
+            node_size,
             node_color,
             edge_color,
             node_labels,
@@ -212,8 +216,24 @@ class Nx3D(ShowBase):
             self._init_keyboard_camera()
         self._init_state_update_task(state_trans_delay, state_trans_func)
 
+    # TODO
+    # setter and getter for node and edge data that propagates to render attributes
+
+    @property
+    def nodes(self):
+        return self.g.nodes
+
+    @property
+    def edges(self):
+        return self.g.edges
+
+    @property
+    def graph(self):
+        return self.g.graph
+
     def _init_render_attrs(
         self,
+        node_size,
         node_color,
         edge_color,
         node_labels,
@@ -229,7 +249,7 @@ class Nx3D(ShowBase):
         graph = self.g
         for n, nd in graph.nodes(data=True):
             nd["color"] = nd.get("color", node_color)
-            nd["shape"] = nd.get("shape", node_color)
+            nd["size"] = nd.get("size", node_size)
             nd["label"] = nd.get("label", self.node_labels.get(n, ""))
             nd["label_color"] = nd.get("label_color", node_label_color)
         for u, v, ed in graph.edges.data():
@@ -251,7 +271,7 @@ class Nx3D(ShowBase):
                     logger.warning(
                         f"initializing positions may take a while for large graphs n>256, len(g)={len(graph)}"
                     )
-                pos_scale = 2.0 * sqrt(len(graph.nodes))
+                pos_scale = get_pos_scale(graph)
                 pos = nx.spring_layout(graph, dim=3, scale=pos_scale)
                 for n, nd in graph.nodes(data=True):
                     nd["pos"] = pos[n]
@@ -311,9 +331,11 @@ class Nx3D(ShowBase):
                 )
             except AttributeError:
                 logger.error("call _init_lights before _init_models")
+                raise
             node.reparentTo(self.render)
             utils.set_color(node, nd["color"])
             node.setPos(*nd["pos"])
+            node.setScale(nd["size"])
             tpid = f"node_{i}_text"
             text, text_ = self._init_panda3d_text(tpid, nd["label"], nd["label_color"])
             text.reparentTo(node)
@@ -332,27 +354,39 @@ class Nx3D(ShowBase):
                 u, v, ek = e
             else:
                 u, v = e
+            pu = graph.nodes[u]["pos"]
+            pv = graph.nodes[v]["pos"]
+            if all(pu == pv):
+                logger.warning(f"nodes {(u, v)} share position")
+                pu = graph.nodes[u]["pos"] = pu + EPS * (1 - np.random.random())
+            dist = linalg.norm(pu - pv)
             pid = f"edge_{i}"
             edge: NodePath = self._init_panda3d_model(
                 pid, self.edge_fp, self.edge_lights
             )
             edge.reparentTo(self.render)
+            tpid = f"edge_{i}_text"
+            text, text_ = self._init_panda3d_text(
+                tpid, graph.edges[e]["label"], graph.edges[e]["label_color"]
+            )
+            text.reparentTo(self.render)
+            graph.edges[e]["model"] = edge
+            graph.edges[e]["text_np"] = text
+            graph.edges[e]["text_tn"] = text_
             utils.set_color(edge, graph.edges[e]["color"])
             # rotate into place
-            pu = graph.nodes[u]["pos"]
-            pv = graph.nodes[v]["pos"]
+            text.setPos(tuple((pu + pv) / 2.0))
             edge.setPos(*pu)
-            dist = linalg.norm(pu - pv)
             edge.setScale(1, 1, dist / 2)  # models have Z size of 2
             d = np.array(pv - pu, dtype=float)
             for ix in np.argwhere(d == 0):
                 d[ix] = EPS
             if d[1] > 0:
                 pitch = -np.arccos(d[2] / dist)
-                assert pitch < 0
+                # assert pitch < 0
             else:
                 pitch = np.arccos(d[2] / dist)
-                assert pitch > 0
+                # assert pitch > 0
             pitch = pitch / pi * 180
             edge.setP(pitch)
             heading = -np.arctan(d[0] / d[1])
@@ -362,15 +396,6 @@ class Nx3D(ShowBase):
                 max_k = max([k for _, _, k in graph.edges])
                 edge.setH(edge, ek / max_k * 90)
             # TODO use lookAt
-            tpid = f"edge_{i}_text"
-            text, text_ = self._init_panda3d_text(
-                tpid, graph.edges[e]["label"], graph.edges[e]["label_color"]
-            )
-            text.reparentTo(self.render)
-            text.setPos(tuple((pu + pv) / 2.0))
-            graph.edges[e]["model"] = edge
-            graph.edges[e]["text_np"] = text
-            graph.edges[e]["text_tn"] = text_
 
     def _init_models(self):
         logger.info("")
